==> Abjad-2.17.tar.gz: Abjad-2.17/abjad/tools/rhythmmakertools/Talea.py <==
# -*- coding: utf-8 -*-
from abjad.tools import datastructuretools
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools.abctools import AbjadValueObject


class Talea(AbjadValueObject):
    '''Talea.

    ..  container:: example

        ::

            >>> talea = rhythmmakertools.Talea(
            ...    counts=(2, 1, 3, 2, 4, 1, 1),
            ...    denominator=16,
            ...    )

    ..  container:: example

        ::

            >>> talea[2]
            NonreducedFraction(3, 16)

    ..  container:: example

        ::

            >>> for nonreduced_fraction in talea[3:9]:
            ...     nonreduced_fraction
            ...
            NonreducedFraction(2, 16)
            NonreducedFraction(4, 16)
            NonreducedFraction(1, 16)
            NonreducedFraction(1, 16)
            NonreducedFraction(2, 16)
            NonreducedFraction(1, 16)

    ..  container:: example

        Taleas can be instantiated without keyword arguments.

        ::

            >>> talea = rhythmmakertools.Talea([1, 2, 3], 8)

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Specifiers'

    __slots__ = (
        '_counts',
        '_denominator',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        counts=(1,),
        denominator=16,
        ):
        from abjad.tools import rhythmmakertools
        counts = tuple(counts)
        assert all(isinstance(x, int) for x in counts)
        self._counts = counts
        assert mathtools.is_nonnegative_integer_power_of_two(denominator)
        self._denominator = denominator

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''Is true when `expr` is a talea with `counts` and `denominator`
        equal to those of this talea. Otherwise false.

        Returns true or false.
        '''
        from abjad.tools import systemtools
        return systemtools.StorageFormatManager.compare(self, expr)

    def __getitem__(self, item):
        r'''Gets non-reduced fraction at `item` cyclically.

        Returns non-reduced fraction or non-reduced fractions.
        '''
        counts = datastructuretools.CyclicTuple(self.counts)
        if isinstance(item, int):
            count = counts[item]
            return mathtools.NonreducedFraction(count, self.denominator)
        elif isinstance(item, slice):
            counts = counts[item]
            result = [mathtools.NonreducedFraction(count, self.denominator)
                for count in counts]
            return result
        raise ValueError(item)

    def __hash__(self):
        r'''Hashes talea.

        Returns integer.
        '''
        from abjad.tools import systemtools
        hash_values = systemtools.StorageFormatManager.get_hash_values(self)
        return hash(hash_values)

    def __iter__(self):
        r'''Iterates talea.

        ..  container:: example

            ::

                >>> talea = rhythmmakertools.Talea(
                ...    counts=(2, 1, 3, 2, 4, 1, 1),
                ...    denominator=16,
                ...    )
                >>> for duration in talea:
                ...     duration
                ...
                Duration(1, 8)
                Duration(1, 16)
                Duration(3, 16)
                Duration(1, 8)
                Duration(1, 4)
                Duration(1, 16)
                Duration(1, 16)

        Yields durations.
        '''
        for count in self.counts:
            duration = durationtools.Duration(count, self.denominator)
            yield duration

    def __len__(self):
        r'''Gets length of talea.

        Returns integer.
        '''
        return len(self.counts)

    ### PUBLIC PROPERTIES ###

    @property
    def counts(self):
        r'''Gets counts of talea.

        Set to integers.

        Returns tuple.
        '''
        return self._counts

    @property
    def denominator(self):
        r'''Gets denominator of talea.

        Set to nonnegative integer power of two.

        Returns nonnegative integer power of two.
        '''
        return self._denominator

==> Abjad-2.18.tar.gz: Abjad-2.18/abjad/tools/rhythmmakertools/Talea.py <==
# -*- coding: utf-8 -*-
from abjad.tools import datastructuretools
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools.abctools import AbjadValueObject


class Talea(AbjadValueObject):
    '''Talea.

    ..  container:: example

        **Example.**

        ::

            >>> talea = rhythmmakertools.Talea(
            ...    counts=(2, 1, 3, 2, 4, 1, 1),
            ...    denominator=16,
            ...    )

    The medieval plural of 'talea' is 'talee'. Abjad documentation
    uses 'taleas' instead.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Specifiers'

    __slots__ = (
        '_counts',
        '_denominator',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        counts=(1,),
        denominator=16,
        ):
        counts = tuple(counts)
        assert all(isinstance(x, int) for x in counts)
        self._counts = counts
        if not mathtools.is_nonnegative_integer_power_of_two(denominator):
            message = 'denominator must be integer power of 2: {!r}.'
            message = message.format(denominator)
            raise Exception(message)
        self._denominator = denominator

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        r'''Is true when `expr` is a talea with `counts` and `denominator`
        equal to those of this talea. Otherwise false.

        ..  container:: example

            **Example.**

            ::

                >>> talea_1 = rhythmmakertools.Talea(
                ...    counts=(1, 2),
                ...    denominator=16,
                ...    )
                >>> talea_2 = rhythmmakertools.Talea(
                ...    counts=(1, 2),
                ...    denominator=16,
                ...    )
                >>> talea_3 = rhythmmakertools.Talea(
                ...    counts=(1, 2),
                ...    denominator=8,
                ...    )

            ::

                >>> talea_1 == talea_1
                True
                >>> talea_1 == talea_2
                True
                >>> talea_1 == talea_3
                False
                >>> talea_2 == talea_1
                True
                >>> talea_2 == talea_2
                True
                >>> talea_2 == talea_3
                False
                >>> talea_3 == talea_1
                False
                >>> talea_3 == talea_2
                False
                >>> talea_3 == talea_3
                True

        Returns true or false.
        '''
        from abjad.tools import systemtools
        return systemtools.StorageFormatManager.compare(self, expr)

    def __getitem__(self, item):
        r'''Gets nonreduced fraction at `item` cyclically.

        ..  container:: example

            **Example 1.** Gets item at index:

            ::

                >>> talea = rhythmmakertools.Talea(
                ...    counts=(2, 1, 3, 2, 4, 1, 1),
                ...    denominator=16,
                ...    )

            ::

                >>> talea[2]
                NonreducedFraction(3, 16)

        ..  container:: example

            **Example 2.** Gets items in slice:

            ::

                >>> for nonreduced_fraction in talea[3:9]:
                ...     nonreduced_fraction
                ...
                NonreducedFraction(2, 16)
                NonreducedFraction(4, 16)
                NonreducedFraction(1, 16)
                NonreducedFraction(1, 16)
                NonreducedFraction(2, 16)
                NonreducedFraction(1, 16)

        Returns nonreduced fraction or nonreduced fractions.
        '''
        counts = datastructuretools.CyclicTuple(self.counts)
        if isinstance(item, int):
            count = counts[item]
            return mathtools.NonreducedFraction(count, self.denominator)
        elif isinstance(item, slice):
            counts = counts[item]
            result = [mathtools.NonreducedFraction(count, self.denominator)
                for count in counts]
            return result
        raise ValueError(item)

    def __hash__(self):
        r'''Hashes talea.

        Returns integer.
        '''
        from abjad.tools import systemtools
        hash_values = systemtools.StorageFormatManager.get_hash_values(self)
        return hash(hash_values)

    def __iter__(self):
        r'''Iterates talea.

        ..  container:: example

            **Example.**

            ::

                >>> talea = rhythmmakertools.Talea(
                ...    counts=(2, 1, 3, 2, 4, 1, 1),
                ...    denominator=16,
                ...    )

            ::

                >>> for duration in talea:
                ...     duration
                ...
                Duration(1, 8)
                Duration(1, 16)
                Duration(3, 16)
                Duration(1, 8)
                Duration(1, 4)
                Duration(1, 16)
                Duration(1, 16)

        Yields durations.
        '''
        for count in self.counts:
            duration = durationtools.Duration(count, self.denominator)
            yield duration

    def __len__(self):
        r'''Gets length.

        ..  container:: example

            **Example.**

            ::

                >>> talea = rhythmmakertools.Talea(
                ...    counts=(2, 1, 3, 2, 4, 1, 1),
                ...    denominator=16,
                ...    )

            ::

                >>> len(talea)
                7

        Defined equal to length of counts.

        Returns nonnegative integer.
        '''
        return len(self.counts)

    ### PUBLIC PROPERTIES ###

    @property
    def counts(self):
        r'''Gets counts.

        ..  container:: example

            **Example.**

            ::

                >>> talea = rhythmmakertools.Talea(
                ...    counts=(2, 1, 3, 2, 4, 1, 1),
                ...    denominator=16,
                ...    )

            ::

                >>> talea.counts
                (2, 1, 3, 2, 4, 1, 1)

        Set to integers.

        Defaults to `(1,)`.

        Returns tuple.
        '''
        return self._counts

    @property
    def denominator(self):
        r'''Gets denominator.

        ..  container:: example

            **Example.**

            ::

                >>> talea = rhythmmakertools.Talea(
                ...    counts=(2, 1, 3, 2, 4, 1, 1),
                ...    denominator=16,
                ...    )

            ::

                >>> talea.denominator
                16

        Set to nonnegative integer power of two.

        Defaults to 16.

        Returns nonnegative integer power of two.
        '''
        return self._denominator
